/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

var carbon = angular.module('carbonApp', ['ngRoute', 'ngTouch']);

// Config
carbon.config(['$routeProvider',
    function ($routeProvider) {
        $routeProvider
            .when('/', {
                templateUrl: 'html/location.html',
                controller: 'locationController'
            })

        .when('/main', {
            templateUrl: 'html/main.html',
            controller: 'mainController'
        })

        .when('/selected', {
            templateUrl: 'html/selected.html',
            controller: 'selectedController'
        })
        
        // I believe notificationSettings is deprecated
        .when('/notificationSettings', {
            templateUrl: 'html/notificationSettings.html',
            controller: 'notificationController'
        })
        
        .when('/settings', {
            templateUrl: 'html/settings.html',
            controller: 'settingsController'
        });
  }]);


angular.module('carbonApp').factory('carbonService', ['$http', '$cacheFactory', '$route',
    function ($http, $cacheFactory, $route) {
        var factory = {},
            cache = $cacheFactory('carbonCache'),
            baseUrl = 'http://api.thedealio.org:443/bond/',
            imageBaseUrl = 'http://api.thedealio.org',
            notificationId = 132479809,
            location = {},
            selectedDeal = {},
            defaultNotification = {
                id: notificationId, // must be a number for Android
                message: "Daily deals on local meals.",
                title: "Check out today's Dealio!",
                autoCancel: true // Remove notification when clicked
            },
            deals = {},
            dailyNotificationTime = "10:00",
            notificationHash = "",
            notifications = {};
        
        if (window.localStorage.getItem("dailyNotificationTime") != null) {
            dailyNotificationTime = window.localStorage.getItem("dailyNotificationTime");
        }

        // Refresh the cache every hour
        setInterval(function() {
            factory.clearCache();
        }, 1800000);  // Update every 30 minutes.
        
        factory.clearCache = function () {
            cache.removeAll();
            $route.reload();
        };
 
        factory.getDealio = function () {
            return $http.get(baseUrl + 'today/' + location.id, {
                cache: cache
            });
        };

        factory.getLocations = function () {
            console.log("GETTING LOCATIONS");
            
            return $http.get(baseUrl + 'locations', {
                cache: cache
            });
        };

        factory.getBusiness = function (business_id) {
            return $http.get(baseUrl + 'business/' + business_id, {
                cache: cache
            });
        };
        
        factory.getNotificationHash = function () {
            return $http.get(baseUrl + 'notifications/hash', {
                cache: cache
            });
        };
        
        factory.getNotifications = function () {
            return $http.get(baseUrl + 'notifications', {
                cache:cache
            });
        };

        factory.timeLeft = function (endDate) {
            if (!angular.isDate(endDate)) {
                endDate = new Date(endDate);
            }

            var timeLeft = Math.floor((endDate - new Date()) / 3600000);
            
            return (timeLeft < 0 ? 0 : timeLeft + 'h');
        };

        factory.openExternalUrl = function (url) {
            if (navigator.app) {
                // Android
                navigator.app.loadUrl(url, { openExternal: true });
            }
            else
            {
                // iOS and others
                window.open(url, '_system');
            }
        };

        factory.setLocation = function (pLocation) {
            location = pLocation;
        };

        factory.setSelectedDeal = function (pSelectedDeal) {
            selectedDeal = pSelectedDeal;
            console.log(pSelectedDeal);
        };

        factory.getSelectedDeal = function () {
            console.log(selectedDeal);
            return deals[selectedDeal];
        };

        factory.setDeals = function (pDeals) {
            deals = pDeals;
        };

        factory.getDeals = function () {
            return deals;
        };

        factory.getDealByIndex = function (pIndex) {
            return deals[i];
        };
        
        factory.setDailyNotificationTime = function (newTime) {
            dailyNotificationTime = newTime;
            console.log("Daily notification time set to " + newTime);
        };

        factory.bindDealioToScope = function ($scope, data) {
            $scope.deals = data;
            
            if (data.length < 1)
            {
                popup.show("Uh Oh!",  "It looks like we can't find any dealios right now in this area.  You should probably check back later.");
                return;
            }

            // TODO: Handle for timezone by importing timezone.js from MIT
            $scope.timeLeft = factory.timeLeft(data[0].end_datetime);

            for (var i in $scope.deals) {
                var deal = $scope.deals[i];
                var tiles = deal.tiles;
                deal.logo_url = imageBaseUrl + deal.logo_url;

                for (var j in tiles)
                    deal[tiles[j].type] = tiles[j].value;
            }
            console.log($scope.deals);
        };

        document.addEventListener('deviceready', function () {
            console.log('Cordova Ready!');
            factory.clearCache();
            pushWoosh.init();
        }, false);
        
        return factory;
  }]);
